@@ -8,5 +8,5 @@ class Labels(object):
     '''
     NLTK labels
     '''
-    default=['GPE','PERSON','ORGANIZATION']
-    geo=['GPE']
\ No newline at end of file
+    default=['GPE','GSP','PERSON','ORGANIZATION']
+    geo=['GPE','GSP']
\ No newline at end of file
